/************************************************************************/
/*                                                                      */
/*      Zoom Page  -  Firefox Extension  -  Main Window                 */
/*                                                                      */
/*      Javascript for Main Window overlay                              */
/*                                                                      */
/*      Copyright (C) 2009-2012  by  DW-dev                             */
/*                                                                      */
/*      Last Edit  -  06 Oct 2012                                       */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* Refer to Firefox source code: mozilla-release\...                    */
/*                                                                      */
/*   browser\base\content\browser-textZoom.js - FullZoom - Firefox 3    */
/*   browser\base\content\browser-fullZoom.js - FullZoom - Firefox 4    */
/*   toolkit\content\viewZoomOverlay.js       - ZoomManager             */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* FullZoom                                                             */
/*                                                                      */
/*   FullZoom.enlarge() calls:                                          */
/*     -  ZoomManager.enlarge()                                         */
/*     -  FullZoom._applySettingToPref()                                */
/*                                                                      */
/*   FullZoom.reduce() calls:                                           */
/*     -  ZoomManager.reduce()                                          */
/*     -  FullZoom._applySettingToPref()                                */
/*                                                                      */
/*   FullZoom.reset() calls:                                            */
/*     -  ZoomManager.reset() or sets ZoomManager.zoom                  */
/*     -  FullZoom._removePref()                                        */
/*                                                                      */
/************************************************************************/

/************************************************************************/
/*                                                                      */
/* ZoomManager                                                          */
/*                                                                      */
/*   ZoomManager.MIN  -  minimum zoom value                             */
/*   ZoomManager.MAX  -  maximum zoom value                             */
/*   ZoomManager.zoomValues[]  -  array of zoom values                  */
/*   ZoomManager.snap()  -  snap to closest zoom value                  */
/*   ZoomManager.zoom  -  current zoom value for selected browser       */
/*   ZoomManager.useFullZoom  -  use full page zoom                     */
/*   ZoomManager.setZoomForBrowser() - redefine to set percent          */
/*   ZoomManager.toggleZoom() - redefine to set mode and set percent    */
/*                                                                      */
/************************************************************************/

var zoomPage =
{
    /********************************************************************/
    
    /* Shared variables */
    
    appInfo: Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo),
    versionComparator: Components.classes["@mozilla.org/xpcom/version-comparator;1"].getService(Components.interfaces.nsIVersionComparator),
    ffVersion: "",
    
    runtime: Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime),
    osPlatform: "",

    prefs: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.zoompage."),
    
    otherPrefs: Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch),
    
    contentPrefs: Components.classes["@mozilla.org/content-pref/service;1"].getService(Components.interfaces.nsIContentPrefService),
    
    zoomLevels: new Array("30-50-67-80-90-100-110-120-133-150-170-200-240-300",
                          "30-50-75-100-125-150-200-300-400",
                          "30-50-75-90-100-110-125-150-200-300-400",
                          "30-50-67-80-90-100-110-120-133-150-170-200-240-300-400",
                          "25-50-75-100-125-150-200-300-400-500",
                          "25-50-75-90-100-110-125-150-200-300-400-500",
                          "25-50-67-80-90-100-110-120-133-150-170-200-240-300-400-500"),
    
    buttonStyle: "",
    
    /********************************************************************/
    
    /* Listen for changes to settings */
    
    prefsObserver:
    {
        register: function()
        {
            /* Query nsIPrefBranch2 interface to observe changes to pref values */
            zoomPage.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
            
            /* Add the observer */
            zoomPage.prefs.addObserver("",this,false);
        },
        
        observe: function(subject,topic,data)
        {
            var i,button;
            var colors = new Array();
            var levelArray = new Array();
            
            if (topic != "nsPref:changed") return;
            
            /* Determine button style */
            
            if (document.getElementById("zoompage-item") != null)  /* button on toolbar */
            {
                if (zoomPage.ffVersion >= +"4.0")
                {
                    zoomPage.buttonStyle = zoomPage.prefs.getCharPref("buttonstyle");
                    
                    document.getElementById("zoompage-style-c").setAttribute("hidden","true");
                    document.getElementById("zoompage-style-n").setAttribute("hidden","true");
                    document.getElementById("zoompage-style-s").setAttribute("hidden","true");
                    document.getElementById("zoompage-style-t").setAttribute("hidden","true");
                    document.getElementById("zoompage-style-m").setAttribute("hidden","true");
                    
                    document.getElementById("zoompage-style-" + zoomPage.buttonStyle).setAttribute("hidden","false");
                    
                    if (zoomPage.buttonStyle == "n" || zoomPage.buttonStyle == "s" || zoomPage.buttonStyle == "t")
                    {
                        colors =  window.getComputedStyle(document.getElementById("zoompage-percent-label-" + zoomPage.buttonStyle),null).
                                  getPropertyValue("color").match(/rgb\((\d+),\s(\d+),\s(\d+)\)/);
                        
                        if (Number(colors[1])+Number(colors[2])+Number(colors[3]) >= 0x80+0x80+0x80)  /* light text means dark theme */
                        {
                            button = document.getElementById("zoompage-zoomout-" + zoomPage.buttonStyle);
                            button.setAttribute("image","chrome://zoompage/skin/zoompage-zoomout-icon-" + zoomPage.buttonStyle + "-d.png");
                            button = document.getElementById("zoompage-zoomin-" + zoomPage.buttonStyle);
                            button.setAttribute("image","chrome://zoompage/skin/zoompage-zoomin-icon-" + zoomPage.buttonStyle + "-d.png");
                        }
                    }
                }
            }
            
            /* Update ZoomManager array of zoom values (relevant to all tabs) */
            
            zoomPage.zoomLevels[7] = zoomPage.prefs.getCharPref("customlevels");
            
            levelArray = zoomPage.zoomLevels[zoomPage.prefs.getIntPref("zoomlevels")].split("-");
            for (i = 0; i < levelArray.length; i++) levelArray[i] = levelArray[i]/100;
            
            delete ZoomManager.MIN;
            ZoomManager.MIN = levelArray[0];
            
            delete ZoomManager.MAX;
            ZoomManager.MAX = levelArray[levelArray.length-1];
            
            delete ZoomManager.zoomValues;
            ZoomManager.zoomValues = levelArray;
            
            /* Update ZoomManager zoom value for selected tab - which will be Add-ons Manager in Firefox 4.0 */
            
            ZoomManager.zoom = ZoomManager.snap(ZoomManager.zoom);
            FullZoom._applySettingToPref();        
            
            /* Update spacer label prefix */
            
            if (document.getElementById("zoompage-item") != null)  /* button on toolbar */
            {
                if (zoomPage.ffVersion == +"3.6")
                {
                    if (zoomPage.prefs.getBoolPref("prefixpercent")) document.getElementById("zoompage-spacer").label = "T888%";
                    else document.getElementById("zoompage-spacer").label = "888%";
                }
                else if (zoomPage.ffVersion >= +"4.0")
                {
                    if (zoomPage.prefs.getBoolPref("prefixpercent")) document.getElementById("zoompage-spacer-label-" + zoomPage.buttonStyle).setAttribute("value","T888%");
                    else document.getElementById("zoompage-spacer-label-" + zoomPage.buttonStyle).setAttribute("value","888%");
                }
            }
            
            /* Update zoom mode and zoom percentage */
            
            zoomPage.setMode(zoomPage.otherPrefs.getBoolPref("browser.zoom.full"));
            zoomPage.setPercent();
        }
    },
    
    /********************************************************************/
    
    /* Load and Unload functions */
    
    onLoad: function()
    {
        var i,navbar,button,child;
        var colors = new Array();
        var levelArray = new Array();
        
        window.removeEventListener("load",zoomPage.onLoad,false);
        
        /* Determine Firefox version and set attribute */
        
        if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"14.0b1") >= 0) zoomPage.ffVersion = "14.0";
        else if (zoomPage.versionComparator.compare(zoomPage.appInfo.version,"4.0b1") >= 0) zoomPage.ffVersion = "4.0";
        else zoomPage.ffVersion = "3.6";
        
        document.getElementById("main-window").setAttribute("zoompage-ffversion",zoomPage.ffVersion);
        
        /* Determine operating system platform and set attribute */
        
        zoomPage.osPlatform = zoomPage.runtime.OS;
        
        document.getElementById("main-window").setAttribute("zoompage-osplatform",zoomPage.osPlatform);
        
        /* Add button to toolbar on first run after installation */
        
        if (zoomPage.prefs.getBoolPref("firstrun"))
        {
            if (document.getElementById("zoompage-item") == null) /* button not on toolbar */
            {
                if (zoomPage.ffVersion >= +"4.0")
                {
                    navbar = document.getElementById("nav-bar");
                    navbar.setAttribute("collapsed","false");
                    navbar.insertItem("zoompage-item",null,null,false);
                    navbar.setAttribute("currentset",navbar.currentSet);
                    document.persist("nav-bar","currentset");
                }
            }
            zoomPage.prefs.setBoolPref("firstrun",false)
        }
        
        /* Replace setZoomForBrowser() in viewZoomOverlay.js with new_setZoomForBrowser() */
        
        ZoomManager.setZoomForBrowser =   
        function new_setZoomForBrowser(aBrowser, aVal)
        {
            if (aVal < this.MIN || aVal > this.MAX)
                throw Components.results.NS_ERROR_INVALID_ARG;
            
            var markupDocumentViewer = aBrowser.markupDocumentViewer;
            
            if (this.useFullZoom)
            {
                markupDocumentViewer.textZoom = 1;
                markupDocumentViewer.fullZoom = aVal;
            }
            else
            {
                markupDocumentViewer.textZoom = aVal;
                markupDocumentViewer.fullZoom = 1;
            }
            
            zoomPage.setPercent();
        },
        
        /* Replace togglezoom() in viewZoomOverlay.js with new_togglezoom() */
        
        ZoomManager.toggleZoom =
        function new_toggleZoom()
        {
            var zoomLevel = this.zoom;
            
            this.useFullZoom = !this.useFullZoom;
            this.zoom = zoomLevel;
            
            zoomPage.setMode(this.useFullZoom);
            zoomPage.setPercent();
        };
        
        /* Add event listeners */
        
        gBrowser.tabContainer.addEventListener("TabOpen",zoomPage.onTabOpen,false);
        gBrowser.tabContainer.addEventListener("TabSelect",zoomPage.onTabSelect,false);
        
        document.getElementById("content").addEventListener("pageshow",zoomPage.onPageShow,false);
        document.getElementById("content").addEventListener("click",zoomPage.onContentClick,false);
        
        window.addEventListener("DOMMouseScroll",zoomPage.onMouseScroll,false); 
        
        if (zoomPage.ffVersion >= +"4.0")
        {
            document.getElementById("navigator-toolbox").addEventListener("drop",zoomPage.onButtonDrop,false);
            document.getElementById("addon-bar").addEventListener("drop",zoomPage.onButtonDrop,false);
        }
        
        /* Button style */
        
        if (document.getElementById("zoompage-item") != null)  /* button on toolbar */
        {
            if (zoomPage.ffVersion == +"3.6")
            {
                zoomPage.prefs.setCharPref("buttonstyle","c");
            }
            else if (zoomPage.ffVersion >= +"4.0")
            {
                zoomPage.buttonStyle = zoomPage.prefs.getCharPref("buttonstyle");
                
                document.getElementById("zoompage-style-c").setAttribute("hidden","true");
                document.getElementById("zoompage-style-n").setAttribute("hidden","true");
                document.getElementById("zoompage-style-s").setAttribute("hidden","true");
                document.getElementById("zoompage-style-t").setAttribute("hidden","true");
                document.getElementById("zoompage-style-m").setAttribute("hidden","true");
                
                document.getElementById("zoompage-style-" + zoomPage.buttonStyle).setAttribute("hidden","false");
                
                if (zoomPage.buttonStyle == "n" || zoomPage.buttonStyle == "s" || zoomPage.buttonStyle == "t")
                {
                    colors =  window.getComputedStyle(document.getElementById("zoompage-percent-label-" + zoomPage.buttonStyle),null).
                              getPropertyValue("color").match(/rgb\((\d+),\s(\d+),\s(\d+)\)/);
                    
                    if (Number(colors[1])+Number(colors[2])+Number(colors[3]) >= 0x80+0x80+0x80)  /* light text means dark theme */
                    {
                        button = document.getElementById("zoompage-zoomout-" + zoomPage.buttonStyle);
                        button.setAttribute("image","chrome://zoompage/skin/zoompage-zoomout-icon-" + zoomPage.buttonStyle + "-d.png");
                        button = document.getElementById("zoompage-zoomin-" + zoomPage.buttonStyle);
                        button.setAttribute("image","chrome://zoompage/skin/zoompage-zoomin-icon-" + zoomPage.buttonStyle + "-d.png");
                    }
                }
            }
        }
        
        /* Zoom Levels */
        
        zoomPage.zoomLevels[7] = zoomPage.prefs.getCharPref("customlevels");
        
        levelArray = zoomPage.zoomLevels[zoomPage.prefs.getIntPref("zoomlevels")].split("-");
        levels = "" + levelArray[0]/100;
        for (i = 1; i < levelArray.length; i++) levels = levels + "," + levelArray[i]/100;
        
        zoomPage.otherPrefs.setIntPref("zoom.minPercent",levelArray[0]);
        zoomPage.otherPrefs.setIntPref("zoom.maxPercent",levelArray[levelArray.length-1]);
        zoomPage.otherPrefs.setCharPref("toolkit.zoomManager.zoomValues",levels);
        
        /* Default Zoom Level */
        
        zoomPage.contentPrefs.setPref(null,"browser.content.full-zoom",zoomPage.prefs.getIntPref("defaultzoom")/100);
        
        /* Do same as onTabOpen() for tabs that do not fire TabOpen events on browser startup */
        
        if (!zoomPage.otherPrefs.getBoolPref("browser.zoom.siteSpecific"))
        {
            for (child = 0; child < gBrowser.tabContainer.childNodes.length; child++)
            {
                ZoomManager.setZoomForBrowser(gBrowser.getBrowserForTab(gBrowser.tabContainer.childNodes[child]),ZoomManager.snap(zoomPage.prefs.getIntPref("defaultzoom")/100));
            }
        }
        
        /* Initialise spacer label prefix */
        
        if (document.getElementById("zoompage-item") != null)  /* button on toolbar */
        {
            if (zoomPage.ffVersion == +"3.6")
            {
                if (zoomPage.prefs.getBoolPref("prefixpercent")) document.getElementById("zoompage-spacer").label = "T888%";
                else document.getElementById("zoompage-spacer").label = "888%";
            }
            else if (zoomPage.ffVersion >= +"4.0")
            {
                if (zoomPage.prefs.getBoolPref("prefixpercent")) document.getElementById("zoompage-spacer-label-" + zoomPage.buttonStyle).setAttribute("value","T888%");
                else document.getElementById("zoompage-spacer-label-" + zoomPage.buttonStyle).setAttribute("value","888%");
            }
        }
        
        /* Initialise zoom mode and zoom percentage */
        
        zoomPage.setMode(zoomPage.otherPrefs.getBoolPref("browser.zoom.full"));
        zoomPage.setPercent();
    },
    
    onUnload: function()
    {
    },
    
    /********************************************************************/
    
    /* Zoom functions */
    
    zoomIn: function()
    {
        FullZoom.enlarge();
        
        zoomPage.setPercent();
    },
    
    zoomOut: function()
    {
        FullZoom.reduce();
        
        zoomPage.setPercent();
    },
    
    zoomReset: function()
    {
        FullZoom.reset();
        
        zoomPage.setPercent();
    },
    
    zoomMode: function(event)
    {
        var zoom;
        
        zoom = ZoomManager.zoom;
        
        zoomPage.otherPrefs.setBoolPref("browser.zoom.full",!zoomPage.otherPrefs.getBoolPref("browser.zoom.full"));  
        zoomPage.setMode(zoomPage.otherPrefs.getBoolPref("browser.zoom.full"));
        
        ZoomManager.zoom = zoom;
        
        zoomPage.setPercent();
    },
    
    setPercent: function()
    {
        if (document.getElementById("zoompage-item") != null)  /* button on toolbar */
        {
            if (zoomPage.ffVersion == +"3.6")
            {
                if (zoomPage.otherPrefs.getBoolPref("browser.zoom.full") || !zoomPage.prefs.getBoolPref("prefixpercent"))
                {
                    document.getElementById("zoompage-zoomdisplay").label = Math.round(ZoomManager.zoom * 100) + "%";
                }
                else
                {
                    document.getElementById("zoompage-zoomdisplay").label = "T" + Math.round(ZoomManager.zoom * 100) + "%";
                }
            }
            else if (zoomPage.ffVersion >= +"4.0")
            {
                if (zoomPage.otherPrefs.getBoolPref("browser.zoom.full") || !zoomPage.prefs.getBoolPref("prefixpercent"))
                {
                    document.getElementById("zoompage-percent-label-" + zoomPage.buttonStyle).setAttribute("value",Math.round(ZoomManager.zoom * 100) + "%");
                    /* using .value does not work after dragging button to Customize window, changing button style, and dragging back to toolbar without any restarts */
                }
                else
                {
                    document.getElementById("zoompage-percent-label-" + zoomPage.buttonStyle).setAttribute("value","T" + Math.round(ZoomManager.zoom * 100) + "%");
                    /* using .value does not work after dragging button to Customize window, changing button style, and dragging back to toolbar without any restarts */
                }
            }
        }
    },
    
    setMode: function(fullpage)
    {
        var tooltip;
        
        if (document.getElementById("zoompage-item") != null)  /* button on toolbar */
        {
            if (fullpage)  /* full page zoom */
            {
                if (zoomPage.ffVersion == +"3.6")
                {
                    document.getElementById("zoompage-zoomdisplay").style.removeProperty("font-style");
                    
                    document.getElementById("zoompage-zoomdisplay").style.removeProperty("color");
                    
                    if (zoomPage.prefs.getBoolPref("leftclick")) tooltip = "zoompage-zoomdisplay-tip-to-lc";
                    else tooltip = "zoompage-zoomdisplay-tip-to";
                    document.getElementById("zoompage-zoomdisplay").tooltip = tooltip;
                }
                else if (zoomPage.ffVersion >= +"4.0")
                {
                    document.getElementById("zoompage-percent-label-" + zoomPage.buttonStyle).style.removeProperty("font-style");
                    
                    document.getElementById("zoompage-percent-label-" + zoomPage.buttonStyle).style.removeProperty("color");
                    
                    if (zoomPage.prefs.getBoolPref("leftclick")) tooltip = "zoompage-zoomdisplay-tip-to-lc";
                    else tooltip = "zoompage-zoomdisplay-tip-to";
                    document.getElementById("zoompage-zoomdisplay-" + zoomPage.buttonStyle).tooltip = tooltip;
                }
            }
            else  /* text only zoom */
            {
                if (zoomPage.ffVersion == +"3.6")
                {
                    if (zoomPage.prefs.getBoolPref("italicpercent")) document.getElementById("zoompage-zoomdisplay").style.setProperty("font-style","italic","");
                    else document.getElementById("zoompage-zoomdisplay").style.removeProperty("font-style");
                    
                    if (zoomPage.prefs.getBoolPref("redpercent")) document.getElementById("zoompage-zoomdisplay").style.setProperty("color","#800000","");
                    else document.getElementById("zoompage-zoomdisplay").style.removeProperty("color");
                    
                    if (zoomPage.prefs.getBoolPref("leftclick")) tooltip = "zoompage-zoomdisplay-tip-fp-lc";
                    else tooltip = "zoompage-zoomdisplay-tip-fp";
                    document.getElementById("zoompage-zoomdisplay").tooltip = tooltip;
                }
                else if (zoomPage.ffVersion >= +"4.0")
                {
                    if (zoomPage.prefs.getBoolPref("italicpercent")) document.getElementById("zoompage-percent-label-" + zoomPage.buttonStyle).style.setProperty("font-style","italic","");
                    else document.getElementById("zoompage-percent-label-" + zoomPage.buttonStyle).style.removeProperty("font-style");
                    
                    if (zoomPage.prefs.getBoolPref("redpercent")) document.getElementById("zoompage-percent-label-" + zoomPage.buttonStyle).style.setProperty("color","#800000","");
                    else document.getElementById("zoompage-percent-label-" + zoomPage.buttonStyle).style.removeProperty("color");
                    
                    if (zoomPage.prefs.getBoolPref("leftclick")) tooltip = "zoompage-zoomdisplay-tip-fp-lc";
                    else tooltip = "zoompage-zoomdisplay-tip-fp";
                    document.getElementById("zoompage-zoomdisplay-" + zoomPage.buttonStyle).tooltip = tooltip;
                }
            }
        }
    },
    
    /********************************************************************/
    
    /* Event handlers */
    
    onDisplayClick: function(event)
    {
        var i,menuPopup,menuItem;
        var levelArray = new Array();
        
        if (event.button == 0)
        {
            if (zoomPage.buttonStyle != "m" && zoomPage.prefs.getBoolPref("leftclick")) zoomPage.zoomReset();
            else
            {
                menuPopup = document.getElementById("zoompage-zoommenupopup");
                
                if (menuPopup.state == "closed")
                {
                    for (i = menuPopup.childNodes.length-1; i >= 0; i--)
                    {
                        menuItem = menuPopup.childNodes[i];
                        menuPopup.removeChild(menuItem);
                    }
                    
                    levelArray = zoomPage.zoomLevels[zoomPage.prefs.getIntPref("zoomlevels")].split("-");
                    
                    for (i = 0; i < levelArray.length; i++)
                    {
                        menuItem = document.createElement("menuitem");
                        menuPopup.appendChild(menuItem);
                        menuItem.setAttribute("label",levelArray[i] + "%");
                        menuItem.setAttribute("oncommand","zoomPage.onMenuCommand(" + levelArray[i] + ")");
                        
                        if (!zoomPage.otherPrefs.getBoolPref("browser.zoom.full"))
                        {
                            if (zoomPage.prefs.getBoolPref("italicpercent")) menuItem.style.setProperty("font-style","italic","");
                            if (zoomPage.prefs.getBoolPref("redpercent")) menuItem.style.setProperty("color","#800000","");
                            if (zoomPage.prefs.getBoolPref("prefixpercent")) menuItem.setAttribute("label","T" + levelArray[i] + "%");
                        }
                        
                        if (levelArray[i] == zoomPage.prefs.getIntPref("defaultzoom")) menuItem.style.setProperty("font-weight","bold","");
                        
                        if (zoomPage.ffVersion == +"3.6")
                        {
                            if (zoomPage.osPlatform == "WINNT")
                                menuItem.style.setProperty("width",(document.getElementById("zoompage-zoomdisplay" + zoomPage.buttonStyle).boxObject.width-5) + "px","");
                            else if (zoomPage.osPlatform == "Linux")
                                menuItem.style.setProperty("width",(document.getElementById("zoompage-zoomdisplay" + zoomPage.buttonStyle).boxObject.width+5) + "px","");
                            else if (zoomPage.osPlatform == "Darwin")
                                menuItem.style.setProperty("width",(document.getElementById("zoompage-zoomdisplay" + zoomPage.buttonStyle).boxObject.width*1.3+1) + "px","");
                        }
                        else if (zoomPage.ffVersion >= +"4.0")
                        {
                            if (zoomPage.osPlatform == "WINNT")
                                menuItem.style.setProperty("width",(document.getElementById("zoompage-zoomdisplay-" + zoomPage.buttonStyle).boxObject.width-4) + "px","");
                            else if (zoomPage.osPlatform == "Linux")
                                menuItem.style.setProperty("width",(document.getElementById("zoompage-zoomdisplay-" + zoomPage.buttonStyle).boxObject.width+6) + "px","");
                            else if (zoomPage.osPlatform == "Darwin")
                                menuItem.style.setProperty("width",(document.getElementById("zoompage-zoomdisplay-" + zoomPage.buttonStyle).boxObject.width*1.3+2) + "px","");
                        }
                    }
                    
                    if (zoomPage.ffVersion == +"3.6")
                    {
                        menuPopup.openPopup(document.getElementById("zoompage-zoomdisplay"),"after_start",-1,0,false,false,null);
                    }
                    else if (zoomPage.ffVersion >= +"4.0")
                    {
                        switch (zoomPage.buttonStyle)
                        {
                            case "c": menuPopup.openPopup(document.getElementById("zoompage-zoomdisplay-c"),"after_start",-1,0,false,false,null); break;
                            case "n": menuPopup.openPopup(document.getElementById("zoompage-zoomdisplay-n"),"after_start",-1,0,false,false,null); break;
                            case "s": menuPopup.openPopup(document.getElementById("zoompage-zoomdisplay-s"),"after_start",-1,0,false,false,null); break;
                            case "t": menuPopup.openPopup(document.getElementById("zoompage-zoomdisplay-t"),"after_start",-1,1,false,false,null); break;
                            case "m": menuPopup.openPopup(document.getElementById("zoompage-zoomdisplay-m"),"after_start",-1,0,false,false,null); break;
                        }
                    }
                }
            }
        }
        else if (event.button == 1) zoomPage.zoomReset();
        else if (event.button == 2)
        {
            zoomPage.zoomMode();
            if (zoomPage.ffVersion >= +"4.0") event.preventDefault();
        }
    },
    
    onDisplayDblClick: function(event)
    {
        event.preventDefault();
    },
    
    onDisplayContextMenu: function(event)
    {
        event.preventDefault();
    },
    
    onMenuCommand: function(value)
    {
        ZoomManager.zoom = ZoomManager.snap(value/100);
        FullZoom._applySettingToPref(); 
        
        zoomPage.setPercent();
    },
    
    onTabOpen: function(event)
    {
        var i;
        
        /* Default ZoomManager zoom value for newly opened tab - if not site specific zoom */
        
        if (!zoomPage.otherPrefs.getBoolPref("browser.zoom.siteSpecific"))
        {
            if (zoomPage.ffVersion == +"3.6")
            {
                window.setTimeout(function (tab) {
                    ZoomManager.setZoomForBrowser(gBrowser.getBrowserForTab(tab),ZoomManager.snap(zoomPage.prefs.getIntPref("defaultzoom")/100));
                },0,event.target);
            }
            else if (zoomPage.ffVersion >= +"4.0")
            {
                ZoomManager.setZoomForBrowser(gBrowser.getBrowserForTab(event.target),ZoomManager.snap(zoomPage.prefs.getIntPref("defaultzoom")/100));
            }
        }
    },
    
    onTabSelect: function(event)
    {
        var i;
        
        /* Update ZoomManager zoom value for newly selected tab - in case zoom levels have been updated */
        
        ZoomManager.zoom = ZoomManager.snap(ZoomManager.zoom);
        FullZoom._applySettingToPref();        
        
        zoomPage.setPercent();
    },
    
    onPageShow: function(event)
    {
        zoomPage.setPercent();
    },
    
    onContentClick: function(event)
    {
        if (event.target.localName == "img" && event.target.parentNode.localName == "body" && event.target.parentNode.childNodes.length == 1)
        {
            /* Viewing image - click on image resizes image to initial size */
            
            zoomPage.setPercent();
        }
    },
    
    onMouseScroll: function(event)
    {
        if (event.ctrlKey)
        {
            if (zoomPage.otherPrefs.getIntPref("mousewheel.withcontrolkey.action") == 3)
            {
                if (event.detail < 0) FullZoom.enlarge(); else FullZoom.reduce();
                
                zoomPage.setPercent();
            }
            
            event.stopPropagation();
            event.preventDefault();
        }
    },
    
    onButtonDrop: function(event)  /* Firefox 4.0 */
    {
        var button;
        var colors = new Array();
        
        if (event.dataTransfer.mozSourceNode.id == "wrapper-zoompage-item")
        {
            /* Determine button style */
            
            zoomPage.buttonStyle = zoomPage.prefs.getCharPref("buttonstyle");
            
            document.getElementById("zoompage-style-c").setAttribute("hidden","true");
            document.getElementById("zoompage-style-n").setAttribute("hidden","true");
            document.getElementById("zoompage-style-s").setAttribute("hidden","true");
            document.getElementById("zoompage-style-t").setAttribute("hidden","true");
            document.getElementById("zoompage-style-m").setAttribute("hidden","true");
            
            document.getElementById("zoompage-style-" + zoomPage.buttonStyle).setAttribute("hidden","false");
            
            if (zoomPage.buttonStyle == "n" || zoomPage.buttonStyle == "s" || zoomPage.buttonStyle == "t")
            {
                colors =  window.getComputedStyle(document.getElementById("zoompage-percent-label-" + zoomPage.buttonStyle),null).
                          getPropertyValue("color").match(/rgb\((\d+),\s(\d+),\s(\d+)\)/);
                
                if (Number(colors[1])+Number(colors[2])+Number(colors[3]) >= 0x80+0x80+0x80)  /* light text means dark theme */
                {
                    button = document.getElementById("zoompage-zoomout-" + zoomPage.buttonStyle);
                    button.setAttribute("image","chrome://zoompage/skin/zoompage-zoomout-icon-" + zoomPage.buttonStyle + "-d.png");
                    button = document.getElementById("zoompage-zoomin-" + zoomPage.buttonStyle);
                    button.setAttribute("image","chrome://zoompage/skin/zoompage-zoomin-icon-" + zoomPage.buttonStyle + "-d.png");
                }
            }
            
            /* Set mode and percent */
            
            zoomPage.setMode(zoomPage.otherPrefs.getBoolPref("browser.zoom.full"));
            zoomPage.setPercent();
        }
    },
};

/************************************************************************/

/* Start Zoom Page */

window.addEventListener("load",zoomPage.onLoad,false);

zoomPage.prefsObserver.register();

/************************************************************************/
